@@ -5,7 +5,7 @@ The module contains a single class, `Perturbations`, which wraps almost all the
 functionality of `hmf` in an easy-to-use way.
 '''
 
-version = '1.2.0'
+version = '1.2.1'
 
 ###############################################################################
 # Some Imports
@@ -23,27 +23,6 @@ from cosmo import Cosmology
 import tools
 from fitting_functions import Fits
 
-# TODO: possibly implement weighting by survey volume: n(M,z1,z2) = \int n(M,z)*V(z) dz / \int V(z) dz
-###############################################################################
-# The Class
-################################################################################
-#    Output Quantities:
-#        sigma:         The mass variance of spheres of the given radius/mass. NOTE: not sigma^2
-#        lnsigma:       The natural logarithm of the inverse of sigma
-#        lnk:           The natural log of the wavenumbers in the power spectrum [h/Mpc]
-#        growth:        The growth factor for the given cosmology and redshift
-#        power:         The natural log of the normalised power spectrum for the given cosmology (at lnk)
-#        n_eff:         Effective spectral index at the radius of a halo
-#        M:             The masses at which analysis is performed. (not log) [M_sun/h]
-#        fsigma:        The multiplicity function, or fitting function at M
-#        dndm:          The comoving number density of halos in mass interval M [h**4/(M_sun * Mpc**3)]
-#        dndlnm:        The comoving number density of halos in log mass interval M [h**3/Mpc**3]
-#        dndlog10m:     The comoving number density of halo in log10 mass interval M [h**3/Mpc**3]
-#        ngtm:          Comoving number density of halos > M [h**3/Mpc**3]
-#        nltm:          Comoving number density of halos < M [h**3/Mpc**3]
-#        mgtm:          Comoving mass density of halos > M [M_sun h**3/Mpc**3]
-#        mltm:          Comoving mass density of halos < M [M_sun h**3/Mpc**3]
-#        how_big:       The requisite size of a simulation box, L, to have at least one halo > M [Mpc/h]
 class Perturbations(object):
     r"""
     An object containing all relevant quantities for the mass function.
@@ -276,9 +255,17 @@ class Perturbations(object):
         """
 
         # First update the cosmology
-        cp = {k:v for k, v in kwargs.iteritems() if k in self._cpdict and v != self._cpdict[k]}
+        cp = {k:v for k, v in kwargs.iteritems() if k in self._cp}
         if cp:
-            self._cpdict.update(cp)
+            true_cp = {}
+            for k, v in cp.iteritems():
+                if k not in self._cpdict:
+                    true_cp[k] = v
+                elif k in self._cpdict:
+                    if v != self._cpdict[k]:
+                        true_cp[k] = v
+
+            self._cpdict.update(true_cp)
 
             # Delete the entries we've used from kwargs
             for k in cp:
@@ -287,27 +274,27 @@ class Perturbations(object):
             # Now actually update the Cosmology class
             self.cosmo_params = Cosmology(default="planck1_base", **self._cpdict)
 
-            if "n" in cp:
+            if "n" in true_cp:
                 del self._unnormalized_power
-            if "sigma_8" in cp:
+            if "sigma_8" in true_cp:
                 del self._power_cdm_0
-            if "delta_c" in cp:
+            if "delta_c" in true_cp:
                 del self.fsigma
 
-            if "omegab" in cp:
+            if "omegab" in true_cp:
                 del self._transfer_original
                 del self._power_cdm_0
                 del self._sigma_0
                 del self._dlnsdlnm
                 del self.dndm
-            if "omegac" in cp:
+            if "omegac" in true_cp:
                 del self._transfer_original
                 del self._power_cdm_0
                 del self._power_0
                 del self._sigma_0
                 del self._dlnsdlnm
                 del self.dndm
-            if "h" in cp or "H0" in cp:
+            if "h" in cp or "H0" in true_cp:
                 del self._transfer_original
                 del self._power_0
                 if "omegab" not in self._cpdict:
@@ -317,13 +304,13 @@ class Perturbations(object):
                     del self._sigma_0
                     del self._dlnsdlnm
                     del self.dndm
-            if "omegab_h2" in cp:
+            if "omegab_h2" in true_cp:
                 del self._transfer_original
                 del self._power_cdm_0
                 del self._sigma_0
                 del self._dlnsdlnm
                 del self.dndm
-            if "omegac_h2" in cp:
+            if "omegac_h2" in true_cp:
                 del self._transfer_original
                 del self._power_cdm_0
                 del self._power_0
@@ -1365,7 +1352,7 @@ class Perturbations(object):
     @property
     def how_big(self):
         """ 
-        Size of simulation volume in which to expect one halo of mass M, ``len=len(M)`` [units :math:`Mpc^{3}h^{-3}`]
+        Size of simulation volume in which to expect one halo of mass M, ``len=len(M)`` [units :math:`Mpch^{-1}`]
         """
 
         return self.ngtm ** (-1. / 3.)
